#include "lru_cache.h"
#include "fifo_cache.h"
#include <pybind11/pybind11.h>



namespace py = pybind11;

PYBIND11_MODULE(TORCH_EXTENSION_NAME, m){

    /* CacheConfig */
    /*-----------------------------------------------------------------------------------------------------------------------------------*/
    py::class_<gpucache::CacheConfig> cfg(m,"CacheConfig");

    cfg.def(py::init<>())
       .def(py::init<gpucache::CacheConfig::CacheEvictStrategy, uint64_t, uint32_t, uint32_t, uint32_t, int8_t,uint32_t>())
       .def_readwrite("strategy", &gpucache::CacheConfig::strategy)
       .def_readwrite("capacity",&gpucache::CacheConfig::capacity)
       .def_readwrite("keySize", &gpucache::CacheConfig::keySize)
       .def_readwrite("valueSize",&gpucache::CacheConfig::valueSize)
       .def_readwrite("maxQueryNum", &gpucache::CacheConfig::maxQueryNum)
       .def_readwrite("deviceId",&gpucache::CacheConfig::deviceId)
       .def_readwrite("dim",&gpucache::CacheConfig::dim);

    py::enum_<gpucache::CacheConfig::CacheEvictStrategy>(cfg,"CacheEvictStrategy")
        .value("LRU",gpucache::CacheConfig::CacheEvictStrategy::LRU)
        .value("LFU",gpucache::CacheConfig::CacheEvictStrategy::LFU)
        .value("FIFO",gpucache::CacheConfig::CacheEvictStrategy::FIFO)
        .export_values();
    /*-----------------------------------------------------------------------------------------------------------------------------------*/

    /* lrucache */
    /*-----------------------------------------------------------------------------------------------------------------------------------*/
    py::class_<gpucache::lrucache::LRUCacheWrapper> lru_cache(m, "LRUCache");

    lru_cache
        .def(py::init<at::Tensor, gpucache::CacheConfig>())
        .def("Get",&gpucache::lrucache::LRUCacheWrapper::Get,"get values for keys,  find_mask return whether each key exists in cache")
        .def("Put",&gpucache::lrucache::LRUCacheWrapper::Put,"put key-value pairs")
        .def("Strategy",&gpucache::lrucache::LRUCacheWrapper::Strategy,"get evict strategy")
        .def("Capacity",&gpucache::lrucache::LRUCacheWrapper::Capacity,"return cache capacity")
        .def("KeySize",&gpucache::lrucache::LRUCacheWrapper::KeySize,"return key size")
        .def("ValueSize",&gpucache::lrucache::LRUCacheWrapper::ValueSize,"return value size")
        .def("MaxQueryNum",&gpucache::lrucache::LRUCacheWrapper::MaxQueryNum,"return max number of keys to get or key-values to put once")
        .def("Clear",&gpucache::lrucache::LRUCacheWrapper::Clear,"clear cache")
        .def("Device",&gpucache::lrucache::LRUCacheWrapper::DeviceId,"return device id")
        .def("Dim",&gpucache::lrucache::LRUCacheWrapper::Dim,"return value dim");

    m.def("NewLRUCache", &gpucache::lrucache::NewLRUCache, "create a lru cache",py::return_value_policy::reference);
    /*-----------------------------------------------------------------------------------------------------------------------------------*/

    /* lrucache */
    /*-----------------------------------------------------------------------------------------------------------------------------------*/
    py::class_<gpucache::fifocache::FIFOCacheWrapper> fifo_cache(m, "FIFOCache");

    fifo_cache
            .def(py::init<at::Tensor, gpucache::CacheConfig>())
            .def("Get",&gpucache::fifocache::FIFOCacheWrapper::Get,"get values for keys,  find_mask return whether each key exists in cache")
            .def("Put",&gpucache::fifocache::FIFOCacheWrapper::Put,"put key-value pairs")
            .def("Strategy",&gpucache::fifocache::FIFOCacheWrapper::Strategy,"get evict strategy")
            .def("Capacity",&gpucache::fifocache::FIFOCacheWrapper::Capacity,"return cache capacity")
            .def("KeySize",&gpucache::fifocache::FIFOCacheWrapper::KeySize,"return key size")
            .def("ValueSize",&gpucache::fifocache::FIFOCacheWrapper::ValueSize,"return value size")
            .def("MaxQueryNum",&gpucache::fifocache::FIFOCacheWrapper::MaxQueryNum,"return max number of keys to get or key-values to put once")
            .def("Clear",&gpucache::fifocache::FIFOCacheWrapper::Clear,"clear cache")
            .def("Device",&gpucache::fifocache::FIFOCacheWrapper::DeviceId,"return device id")
            .def("Dim",&gpucache::fifocache::FIFOCacheWrapper::Dim,"return value dim");

    m.def("NewFIFOCache", &gpucache::fifocache::NewFIFOCache, "create a fifo cache",py::return_value_policy::reference);
    /*-----------------------------------------------------------------------------------------------------------------------------------*/


    
}


