#pragma once
#include <torch/extension.h>
#include "cache.h"


namespace gpucache {
    namespace lrucache {

        class LRUCacheWrapper {
        public:
            LRUCacheWrapper(at::Tensor t, CacheConfig cfg);

            ~LRUCacheWrapper();

            std::pair<torch::Tensor, torch::Tensor> Get(uint32_t num_query, const torch::Tensor queries);

            void Put(uint32_t num_query, const torch::Tensor keys, const torch::Tensor values);

            CacheConfig::CacheEvictStrategy Strategy();

            uint64_t Capacity();

            uint32_t KeySize();

            uint32_t ValueSize();

            uint32_t MaxQueryNum();

            uint64_t DeviceId();

            uint32_t Dim();

            void Clear();

        private:
            void *lru_cache;
            c10::ScalarType dtype; // value dtype
            c10::ScalarType kdtype; // key dtype
            bool key_is_int32; // only support int32 and int64
            CacheConfig cache_cfg;
        };

        std::unique_ptr<LRUCacheWrapper> NewLRUCache(at::Tensor t, CacheConfig cfg);
    } // namespace lrucache
} // namespace gpucache