#include "utils.cuh"

namespace gpucache {


    __device__ ThreadCtx::ThreadCtx() {
        auto global_thread_id = blockIdx.x * blockDim.x + threadIdx.x;
        global_warp_idx = global_thread_id / warpsize;
        block_warp_idx = threadIdx.x / warpsize;
        lane_id = threadIdx.x % warpsize;
        num_warps = blockDim.x * gridDim.x / warpsize;
    }


    __device__ WarpMutex::WarpMutex() : flag(0) {}

    __device__ void WarpMutex::Lock(ThreadCtx &ctx, uint32_t bucket_id) {
        if (ctx.lane_id == 0) {
            while (atomicCAS(&flag, 0, 1) != 0) {}
        }
        __threadfence();
        __syncwarp();
    }

    __device__ void WarpMutex::UnLock(ThreadCtx &ctx) {
        __syncwarp();
        __threadfence();
        if (ctx.lane_id == 0) {
            atomicExch(&flag, 0);
        }
    }


    __global__ void initLocks(uint32_t n_bucket, void *bucketMutexes) {
        uint32_t global_thread_idx = blockDim.x * blockIdx.x + threadIdx.x;
        if (global_thread_idx < n_bucket) {
            new(reinterpret_cast<WarpMutex *>(bucketMutexes) + global_thread_idx)
                    WarpMutex();
        }
    }

    __global__ void checkLocks(uint32_t n_bucket, void *bucketMutexes) {
        uint32_t global_thread_idx = blockDim.x * blockIdx.x + threadIdx.x;
        // printf("thread %u CUDA_CHECK lock\n",global_thread_idx);
        if (global_thread_idx < n_bucket) {
            auto mutex =
                    reinterpret_cast<WarpMutex *>(bucketMutexes) + global_thread_idx;
            if (mutex->flag != 0u && mutex->flag != 1u) {
                printf("bucket id %u not equal 0 or 1, is %u\n", global_thread_idx,
                       mutex->flag);
            }
        }
    }


}


