#pragma once
#include <cuda_runtime.h>

namespace gpucache{
    constexpr unsigned int warpFullMask = 0xFFFFFFFF;
    constexpr unsigned int defaultBlockX = 256;
    constexpr unsigned int warpsize = 32;
    constexpr unsigned int defaultNumWarpsPerBlock = defaultBlockX / warpsize;
    // bucket_id + key
    constexpr unsigned int uint32SharedMemorySize = 2 * sizeof(uint32_t) * defaultNumWarpsPerBlock * warpsize;
    constexpr unsigned int uint64SharedMemorySize =
            (sizeof(uint64_t) + sizeof(uint32_t)) * defaultNumWarpsPerBlock * warpsize;


    struct ThreadCtx {
        __device__ ThreadCtx();

        uint32_t global_warp_idx;
        uint32_t block_warp_idx;
        uint32_t num_warps;
        uint32_t lane_id;
    };

    struct WarpMutex {
    public:
        __device__ WarpMutex();

        ~WarpMutex() = default;

        WarpMutex(const WarpMutex &) = delete;

        WarpMutex &operator=(const WarpMutex &) = delete;

        WarpMutex(WarpMutex &&) = delete;

        WarpMutex &operator=(WarpMutex &&) = delete;

        __device__ void Lock(ThreadCtx &ctx, uint32_t bucket_id);

        __device__ void UnLock(ThreadCtx &ctx);

        //    private:
        uint32_t flag;
    };

    __global__ void initLocks(uint32_t n_bucket, void *bucketMutexes);

    __global__ void checkLocks(uint32_t n_bucket, void *bucketMutexes);




}



